!--------------------------------------------------------------------------!
! The Phantom Smoothed Particle Hydrodynamics code, by Daniel Price et al. !
! Copyright (c) 2007-2019 The Authors (see AUTHORS)                        !
! See LICENCE file for usage and distribution conditions                   !
! http://phantomsph.bitbucket.io/                                          !
!--------------------------------------------------------------------------!
!+
!  MODULE: kernel
!
!  DESCRIPTION:
!   This module implements the Wendland 2/3D C^2 kernel
!   DO NOT EDIT - auto-generated by kernels.py
!
!  REFERENCES: None
!
!  OWNER: Daniel Price
!
!  $Id: fecd76f08631bf7058052e45216f2ae088fcfc84 $
!
!  RUNTIME PARAMETERS: None
!
!  DEPENDENCIES: physcon
!
!  GENERATED: 2014-11-20 11:03:54.085594
!+
!--------------------------------------------------------------------------
module kernel
 use physcon, only:pi
 implicit none
 character(len=17), public :: kernelname = 'Wendland 2/3D C^2'
 real, parameter, public  :: radkern  = 2.
 real, parameter, public  :: radkern2 = 4.
 real, parameter, public  :: cnormk = 1.3125/pi
 real, parameter, public  :: wab0 = 1., gradh0 = -3.*wab0
 real, parameter, public  :: dphidh0 = 1.5
 real, parameter, public  :: cnormk_drag = 105./(64.*pi)
 real, parameter, public  :: hfact_default = 1.3
 real, parameter, public  :: av_factor = 10./9.

contains

pure subroutine get_kernel(q2,q,wkern,grkern)
 real, intent(in)  :: q2,q
 real, intent(out) :: wkern,grkern

 !--Wendland 2/3D C^2
 if (q < 2.) then
    wkern  = (0.5*q - 1.)**4*(2.*q + 1.)
    grkern = q*(0.625*q2*q - 3.75*q2 + 7.5*q - 5.)
 else
    wkern  = 0.
    grkern = 0.
 endif

end subroutine get_kernel

pure elemental real function wkern(q2,q)
 real, intent(in) :: q2,q

 if (q < 2.) then
    wkern = (0.5*q - 1.)**4*(2.*q + 1.)
 else
    wkern = 0.
 endif

end function wkern

pure elemental real function grkern(q2,q)
 real, intent(in) :: q2,q

 if (q < 2.) then
    grkern = q*(0.625*q2*q - 3.75*q2 + 7.5*q - 5.)
 else
    grkern = 0.
 endif

end function grkern

pure subroutine get_kernel_grav1(q2,q,wkern,grkern,dphidh)
 real, intent(in)  :: q2,q
 real, intent(out) :: wkern,grkern,dphidh
 real :: q4, q6

 if (q < 2.) then
    q4 = q2*q2
    q6 = q4*q2
    wkern  = (0.5*q - 1.)**4*(2.*q + 1.)
    grkern = q*(0.625*q2*q - 3.75*q2 + 7.5*q - 5.)
    dphidh = -0.09375*q6*q + 0.8203125*q6 - 2.625*q4*q + 3.28125*q4 - 2.625*q2 + 1.5
 else
    wkern  = 0.
    grkern = 0.
    dphidh = 0.
 endif

end subroutine get_kernel_grav1

pure subroutine kernel_softening(q2,q,potensoft,fsoft)
 real, intent(in)  :: q2,q
 real, intent(out) :: potensoft,fsoft
 real :: q4, q6

 if (q < 2.) then
    q4 = q2*q2
    q6 = q4*q2
    potensoft = 3.*q6*q/256. - 15.*q6/128. + 7.*q4*q/16. - 21.*q4/32. + 7.*q2/8. - 3./2.
    fsoft     = q*(21.*q4*q - 180.*q4 + 560.*q2*q - 672.*q2 + 448.)/256.
 else
    potensoft = -1./q
    fsoft     = 1./q2
 endif

end subroutine kernel_softening

!------------------------------------------
! double-humped version of the kernel for
! use in drag force calculations
!------------------------------------------
pure elemental real function wkern_drag(q2,q)
 real, intent(in) :: q2,q

 !--double hump Wendland 2/3D C^2 kernel
 if (q < 2.) then
    wkern_drag = q2*(0.5*q - 1.)**4*(2.*q + 1.)
 else
    wkern_drag = 0.
 endif

end function wkern_drag

end module kernel
